
/******************************************************************************
* MODULE     : matrix.hpp
* DESCRIPTION: Generic matrix
* COPYRIGHT  : (C) 2004  Olivier Ruatta
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_MATRIX_HPP
#define __MML_MATRIX_HPP
#include "iterator.hpp"
#include "block.hpp"
namespace mml {
#define TMPL_DEF template<typename C, typename M=dense<naive> >
#define TMPL template<typename C, typename M>
#define Matrix matrix<C,M>
TMPL class matrix;

struct one_nat {
  nat i;
  inline one_nat (int i2): i (i2) {}
};

struct pair_nat {
  nat i, j;
  inline pair_nat (nat i2, nat j2): i (i2), j (j2) {}
  inline friend pair_nat operator , (one_nat i, nat j) {
    return pair_nat (i.i, j); }
};

/******************************************************************************
* Abstract interface for generic matrices
******************************************************************************/

TMPL_DEF class matrix_rep REP_STRUCT {};

TMPL_DEF
class matrix {
INDIRECT_PROTO_2 (matrix, matrix_rep, C, M)
public:
  matrix ();
  matrix (nat i, nat j);
  matrix (const C& x);
  matrix (const iterator<C>& it);
  C operator () (nat i, nat j) const;
  C& ref (nat i, nat j);
};
INDIRECT_IMPL_2 (matrix, matrix_rep, typename C, C, typename M, M)

TMPL nat rows (const Matrix& m);
TMPL nat cols (const Matrix& m);

/******************************************************************************
* Matrix iteration
******************************************************************************/

TMPL_DEF
class matrix_iterator_rep: public iterator_rep<C> {
  Matrix m;  // the matrix to traverse
  nat i;     // current row
  nat j;     // current column
  nat nc;
protected:
  bool is_busy () { return (i*cols(m)+j < rows (m)*cols (m) ); }
  void advance () { if (j < cols(m)) j++; else { j=0; i++; }  }
  C current () { return m (i, j); }
public:
  matrix_iterator_rep (const Matrix& m2, nat c = M::def_cols):
    m(m2), i(0), j(0), nc(c) {}
};

TMPL iterator<C>
iterate (const Matrix& m) {
  return iterator<C> (new matrix_iterator_rep<C,M> (m));
}

/******************************************************************************
* Generic output routines for matrices
******************************************************************************/

TMPL content
as_content (const Matrix& m) {
  int i, j, nr= rows(m), nc= cols(m);
  content v;
  for (i=0; i<nr; i++) {
    content h;
    for (j=0; j<nc; j++)
      h << as_content (m (i, j));
    v << tuple (h);
  }
  return sqtuple (vtuple (v));
}

PRINT_SUGAR(TMPL,Matrix)

#undef TMPL_DEF
#undef TMPL
#undef Matrix
} //namespace mml

#endif //__MML_MATRIX_HPP
