
/******************************************************************************
* MODULE     : polynomial_test.cc
* DESCRIPTION: Test polynomials
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "integer.hpp"
#include "dense_polynomial.hpp"
#include "karatsuba.hpp"
using namespace mml;

#define Scalar integer
#define Polynomial polynomial<Scalar,dense<karatsuba<2> > >

Polynomial
test (nat deg) {
  Polynomial P (1, deg);
  for (nat i=0; i<deg; i++)
    P += Polynomial (1, i);
  return P;
}

int
main () {
  Polynomial P= seq (Scalar (1), Scalar (5), Scalar (2));
  Polynomial Q (7, 3);
  cout << "P\t= " << P << "\n";
  cout << "Q\t= " << Q << "\n";
  cout << "P+Q\t= " << P+Q << "\n";
  cout << "P-Q\t= " << P-Q << "\n";
  cout << "Q-P\t= " << Q-P << "\n";
  cout << "P*Q\t= " << P*Q << "\n";
  cout << "2*P\t= " << 2*P << "\n";
  cout << "P*2\t= " << P*2 << "\n";
  cout << "P/3\t= " << P/3 << "\n";
  cout << "P+=Q\t= " << (P+=Q) << "\n";
  cout << "P-=Q\t= " << (P-=Q) << "\n";
  //cout << "P*=Q\t= " << (P*=Q) << "\n";
  //cout << "P/=Q\t= " << (P/=Q) << "\n";
  cout << "P*=11\t= " << (P*=11) << "\n";
  cout << "P/=11\t= " << (P/=11) << "\n";
  cout << "P=Q\t= " << (P==Q) << "\n";
  cout << "P!=Q\t= " << (P!=Q) << "\n";
  cout << "P<Q\t= " << (P<Q) << "\n";
  cout << "P<=Q\t= " << (P<=Q) << "\n";
  cout << "P>Q\t= " << (P>Q) << "\n";
  cout << "P>=Q\t= " << (P>=Q) << "\n";
  cout << "P...\t= " << iterate (P) << "\n";

  cout << "\n";
  for (nat i=0; i<6; i++)
    for (nat j=i; j<6; j++) {
      Polynomial P= test (i);
      Polynomial Q= test (j);
      cout << "(" << P << ") (" << Q << ") = " << P*Q << "\n";
    }

  return 0;
}
