
/******************************************************************************
* MODULE     : context.hpp
* DESCRIPTION: Contexts for symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __CONTEXT_HPP
#define __CONTEXT_HPP
#include "symbolic.hpp"
#include "table.hpp"
namespace mml {

class context;
class context_rep: public rep_struct {
public:
  symbolic id;

public:
  inline context_rep (const symbolic& id2): id (id2) {}
  inline virtual ~context_rep () {}
  inline context me () const;

  // find out which context implements an operation and performing it
  virtual bool extends (const context& c) const;
  virtual context least_common_context (const context& c);
  virtual symbolic eval (const symbolic& s) const;
  virtual symbolic eval_atomic (const symbolic& s) const;
  virtual symbolic eval_compound (const symbolic& s) const;
  virtual symbolic eval (const symbolic& s, bool recursive) const;

  // common operations
  virtual symbolic add (const symbolic& s1, const symbolic& s2) const;
  virtual symbolic sub (const symbolic& s1, const symbolic& s2) const;
  virtual symbolic neg (const symbolic& s1) const;
  virtual symbolic mul (const symbolic& s1, const symbolic& s2) const;
  virtual symbolic div (const symbolic& s1, const symbolic& s2) const;
  virtual symbolic pow (const symbolic& s1, const symbolic& s2) const;

  friend class context;
};

class context {
INDIRECT_PROTO (context, context_rep)
public:
  inline context (): rep (new context_rep (symbolic (CONTEXT_FREE))) {}
  inline friend nat hash (const context& c) { return hash (c.rep->id); }
  inline bool operator == (const context& c) const {
    return rep->id == c.rep->id; }
  inline bool operator != (const context& c) const {
    return rep->id != c.rep->id; }
  inline bool operator <  (const context& c) const {
    return rep->id <  c.rep->id; }
  inline bool operator <= (const context& c) const {
    return rep->id <= c.rep->id; }
  inline bool operator >  (const context& c) const {
    return rep->id >  c.rep->id; }
  inline bool operator >= (const context& c) const {
    return rep->id >= c.rep->id; }

  inline friend context least_common_context (
    const context& c1, const context& c2) {
    return c1.rep->least_common_context (c2); }
  inline friend symbolic eval (const context& c, const symbolic& s, bool rec) {
    return c.rep->eval (s, rec); }
  friend class context_rep;
};
INDIRECT_IMPL (context, context_rep)

inline context context_rep::me () const {
  return context (const_cast<context_rep*> (this), true); }

inline context free_context () { return context (); }
context scalar_context (const context& base);

} // namespace mml
#endif // __CONTEXT_HPP
