
/******************************************************************************
* MODULE     : symbolic_impl.hpp
* DESCRIPTION: Extra routines for the implementation of symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __SYMBOLIC_IMPL_HPP
#define __SYMBOLIC_IMPL_HPP
#include "symbolic.hpp"
#include "table.hpp"
namespace mml {

typedef table<symbolic,symbolic> symbolic_commutative;

symbolic scalar_mul (const symbolic& sc, const symbolic& s1);

inline bool
is_scalar (const symbolic& s) {
  symbolic_type t= s->type ();
  return t == SYMBOLIC_INTEGER || t == SYMBOLIC_RATIONAL;
}

integer as_integer (const symbolic& s);
symbolic integer_add (const symbolic& s1, const symbolic& s2);
symbolic integer_sub (const symbolic& s1, const symbolic& s2);
symbolic integer_neg (const symbolic& s1);
symbolic integer_mul (const symbolic& s1, const symbolic& s2);
symbolic integer_div (const symbolic& s1, const symbolic& s2);

rational as_rational (const symbolic& s);
symbolic rational_add (const symbolic& s1, const symbolic& s2);
symbolic rational_sub (const symbolic& s1, const symbolic& s2);
symbolic rational_neg (const symbolic& s1);
symbolic rational_mul (const symbolic& s1, const symbolic& s2);
symbolic rational_div (const symbolic& s1, const symbolic& s2);

symbolic_commutative as_symbolic_sum (const symbolic& s);
symbolic sum_add (const symbolic& s1, const symbolic& s2);
symbolic sum_sub (const symbolic& s1, const symbolic& s2);
symbolic sum_neg (const symbolic& s1);
symbolic sum_scalar_mul (const symbolic& sc, const symbolic& s1);

symbolic product_mul (const symbolic& s1, const symbolic& s2);
symbolic product_div (const symbolic& s1, const symbolic& s2);
symbolic product_pow (const symbolic& s1, const symbolic& p);

} // namespace mml
#endif // __SYMBOLIC_HPP
