
/******************************************************************************
* MODULE     : context_basic.cpp
* DESCRIPTION: Basic context with elementary simplifications for printing
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "context_base.hpp"
#include "symbolic_impl.hpp"
namespace mml {

class context_basic_rep: public context_base_rep {
public:
  inline context_basic_rep (const context& base):
    context_base_rep (base, symbolic_compound ("basic", base->id)) {}

  symbolic add (const symbolic& s1, const symbolic& s2) const {
    if (s1 == SYMB_ZERO) return s2;
    if (s2 == SYMB_ZERO) return s1;
    return base->add (s1, s2);
  }
  symbolic sub (const symbolic& s1, const symbolic& s2) const {
    if (s2 == SYMB_ZERO) return s1;
    if (s1 == SYMB_ZERO) return -s2;
    return base->sub (s1, s2);
  }
  symbolic neg (const symbolic& s1) const {
    if (s1 == SYMB_ZERO) return SYMB_ZERO;
    return base->neg (s1);
  }
  symbolic mul (const symbolic& s1, const symbolic& s2) const {
    if (s1 == SYMB_ZERO) return SYMB_ZERO;
    if (s2 == SYMB_ZERO) return SYMB_ZERO;
    if (s1 == SYMB_ONE) return s2;
    if (s2 == SYMB_ONE) return s1;
    return base->mul (s1, s2);
  }
  symbolic div (const symbolic& s1, const symbolic& s2) const {
    if (s1 == SYMB_ZERO) return SYMB_ZERO;
    if (s2 == SYMB_ONE) return s1;
    return base->div (s1, s2);
  }
  symbolic pow (const symbolic& s1, const symbolic& s2) const {
    if (s1 == SYMB_ZERO) return SYMB_ZERO;
    if (s2 == SYMB_ZERO) return SYMB_ONE;
    if (s1 == SYMB_ONE) return SYMB_ONE;
    if (s2 == SYMB_ONE) return s1;
    return base->pow (s1, s2);
  }
};

context
basic_context (const context& base) {
  return new context_basic_rep (base);
}

} // namespace mml
