
/******************************************************************************
* MODULE     : context_commutative.cpp
* DESCRIPTION: Context for commutative field expressions with powers
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "context_base.hpp"
#include "symbolic_impl.hpp"
namespace mml {

inline symbolic tag (const symbolic& s, const context& c) {
  // FIXME: We should tag the symbolic expression to belong to the context c
  return s;
}

class context_commutative_rep: public context_base_rep {
  selector sel;

public:
  inline context_commutative_rep (const context& base, const selector& sel2):
    context_base_rep (base, symbolic_compound ("commutative", base->id)),
    sel (sel2) {} // FIXME: also take selector into account in id

  // Selection of ground terms and tagging of expressions
  symbolic eval_atomic (const symbolic& s) {
    if (sel (s)) return tag (s, me ());
    else return base->eval_atomic (s);
  }
  symbolic eval_compound (const symbolic& s) {
    if (sel (s)) return tag (s, me ());
    else return base->eval_compound (s);
  }

  // Arithmetic operations in commutative context
  symbolic add (const symbolic& s1, const symbolic& s2) const {
    if (s1->in (base) && s2->in (base))
      return base->add (s1, s2);
    return sum_add (s1, s2);
  }
  symbolic sub (const symbolic& s1, const symbolic& s2) const {
    if (s1->in (base) && s2->in (base))
      return base->sub (s1, s2);
    return sum_sub (s1, s2);
  }
  symbolic neg (const symbolic& s1) const {
    if (s1->in (base))
      return base->neg (s1);
    return base->neg (s1);
  }
  symbolic mul (const symbolic& s1, const symbolic& s2) const {
    if (s1->in (base) && s2->in (base))
      return base->mul (s1, s2);
    return product_mul (s1, s2);
  }
  symbolic div (const symbolic& s1, const symbolic& s2) const {
    if (s1->in (base) && s2->in (base))
      return base->div (s1, s2);
    return product_div (s1, s2);
  }
  symbolic pow (const symbolic& s1, const symbolic& s2) const {
    return product_pow (s1, s2);
  }
};

context
commutative_context (const context& base, const selector& sel) {
  return new context_commutative_rep (base, sel);
}

} // namespace mml
