
/******************************************************************************
* MODULE     : symbolic.cpp
* DESCRIPTION: Symbolic expressions
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic_aux.hpp"
#include "context.hpp"
namespace mml {

context symbolic_rep::get_context () const {
  return free_context (); }
bool symbolic_rep::in (const context& c) const {
  return c->extends (get_context ()); }

bool symbolic_rep::is_atomic () { return mml::is_atomic (flatten (false)); }
string symbolic_rep::leaf () { return L (flatten (false)); }
nat symbolic_rep::arity () { return N (flatten (false)); }
symbolic symbolic_rep::child (nat i) { return flatten (false)[i]; }

// FIXME: the arithmetic types should be taken from the context
add_type symbolic_rep::get_add_type () const { return ADD_GENERIC; }
mul_type symbolic_rep::get_mul_type () const { return MUL_GENERIC; }
pow_type symbolic_rep::get_pow_type () const { return POW_GENERIC; }
//add_type symbolic_rep::get_add_type () const { return ADD_COMMUTATIVE; }
//mul_type symbolic_rep::get_mul_type () const { return MUL_COMMUTATIVE; }
//pow_type symbolic_rep::get_pow_type () const { return POW_EXPO_LINEAR; }

symbolic::symbolic (symbolic_rep* rep2): rep (memorize (rep2)) {}
symbolic::~symbolic () { forget (rep); DEC_COUNT (rep); }
symbolic& symbolic::operator = (const symbolic& x) {
  INC_COUNT (x.rep); forget (rep); DEC_COUNT (rep);
  rep=x.rep; return *this; }

/******************************************************************************
* apply==symbolic_compound for the moment
******************************************************************************/

symbolic apply (const vector<symbolic>& fun_args) {
  return symbolic_compound (fun_args); }
symbolic apply (const symbolic& fun, const vector<symbolic>& args) {
  return symbolic_compound (fun, args); }
symbolic apply (const symbolic& fun) {
  return symbolic_compound (fun); }
symbolic apply (const symbolic& fun, const symbolic& s1) {
  return symbolic_compound (fun, s1); }
symbolic apply (const symbolic& fun, const symbolic& s1, const symbolic& s2) {
  return symbolic_compound (fun, s1, s2); }

} // namespace mml
