
/******************************************************************************
* MODULE     : symbolic_rational.cpp
* DESCRIPTION: Symbolic rationals
* COPYRIGHT  : (C) 2004  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include "symbolic_impl.hpp"
#include "symbolic_aux.hpp"
#include "rational.hpp"
namespace mml {
symbolic_rep* symbolic_integer (const integer& i);

class symbolic_rational_rep: public symbolic_rep {
public:
  rational data;

protected:
  // accessors and syntactic identity
  symbolic_type type () const { return SYMBOLIC_RATIONAL; }
  nat hash () const { return mml::hash (data); }
  bool identical (symbolic_rep* rep) const {
    if (rep->type() != SYMBOLIC_RATIONAL) return false;
    return data == ((symbolic_rational_rep*) rep)->data; }
  symbolic flatten (bool recursive) const {
    return symbolic_compound ("/",
			      mml::flatten (num (data), recursive),
			      mml::flatten (den (data), recursive)); }

  // arithmetic
  add_type get_add_type () const { return ADD_RATIONAL; }
  mul_type get_mul_type () const { return MUL_RATIONAL; }
  pow_type get_pow_type () const { return POW_EXPO_LINEAR; }

public:
  symbolic_rational_rep (const rational& data2): data (data2) {}
};

/******************************************************************************
* Constructors and conversions
******************************************************************************/

symbolic::symbolic (const rational& r) {
  if (den (r) == 1) rep= memorize (symbolic_integer (num (r)));
  else rep= memorize (new symbolic_rational_rep (r));
}

rational
as_rational (const symbolic& s) {
  if (s->type () == SYMBOLIC_INTEGER)
    return rational (as_integer (s));
  assert (s->type () == SYMBOLIC_RATIONAL);
  return ((symbolic_rational_rep*) s.rep)->data;
}

/******************************************************************************
* Arithmetic
******************************************************************************/

symbolic
rational_add (const symbolic& s1, const symbolic& s2) {
  return as_rational (s1) + as_rational (s2);
}

symbolic
rational_sub (const symbolic& s1, const symbolic& s2) {
  return as_rational (s1) - as_rational (s2);
}

symbolic
rational_neg (const symbolic& s1) {
  return -as_rational (s1);
}

symbolic
rational_mul (const symbolic& s1, const symbolic& s2) {
  return as_rational (s1) * as_rational (s2);
}

symbolic
rational_div (const symbolic& s1, const symbolic& s2) {
  return as_rational (s1) / as_rational (s2);
}

} // namespace mml
